@@ -53,8 +53,10 @@ var gLastValidURLStr = "";
 var gLastValidURL = null;
 var gHaveUpdatedToolbarState = false;
 var gClickSelectsAll = false;
+var gClickAtEndSelects = false;
 var gIgnoreFocus = false;
 var gIgnoreClick = false;
+var gURIFixup = null;
 
 var pref = null;
 
@@ -123,9 +125,75 @@ const gHomepagePrefListener =
     if (topic != "nsPref:changed")
       return;
 
-    var homePage = getHomePage();
-    if (homePage)
-      setTooltipText("home-button", homePage);
+    updateHomeButtonTooltip();
+  }
+};
+
+// popup window permission change listener
+const gPopupPermListener = {
+
+  observe: function(subject, topic, data) {
+    if (topic == "popup-perm-close") {
+      // close the window if we're a popup and our opener's URI matches
+      // the URI in the notification
+      var popupOpenerURI = maybeInitPopupContext();
+      if (popupOpenerURI) {
+        const IOS = Components.classes["@mozilla.org/network/io-service;1"]
+                    .getService(Components.interfaces.nsIIOService);
+        closeURI = IOS.newURI(data, null, null);
+        if (closeURI.host == popupOpenerURI.host)
+          window.close();
+      }
+    }
+  }
+};
+
+const POPUP_TYPE = 2;
+const gPopupPrefListener =
+{
+  domain: "dom.disable_open_during_load",
+  observe: function(subject, topic, prefName)
+  {        
+    if (topic != "nsPref:changed" || prefName != this.domain)
+      return;
+
+    var browsers = getBrowser().browsers;
+    var policy = pref.getBoolPref(prefName);
+
+    if (policy && pref.getBoolPref("privacy.popups.first_popup"))
+      pref.setBoolPref("privacy.popups.first_popup", false);
+
+    var hosts = [];
+
+    var popupManager = Components.classes["@mozilla.org/PopupWindowManager;1"]
+                                 .getService(Components.interfaces.nsIPopupWindowManager);
+    
+    var enumerator = popupManager.getEnumerator();
+    var count=0;
+    while (enumerator.hasMoreElements()) {
+      var permission = enumerator.getNext()
+                                 .QueryInterface(Components.interfaces.nsIPermission);
+      if (permission.capability == policy)
+        hosts[permission.host] = permission.host;
+    }
+
+    var popupIcon = document.getElementById("popupIcon");
+    
+    if (!policy) {
+      for (var i = 0; i < browsers.length; i++) {
+        if (browsers[i].popupDomain in hosts)
+          break;
+        browsers[i].popupDomain = null;
+        popupIcon.hidden = true;
+      }
+    } else {
+      for (i = 0; i < browsers.length; i++) {
+        if (browsers[i].popupDomain in hosts) {
+          browsers[i].popupDomain = null;
+          popupIcon.hidden = true;
+        }
+      }
+    }
   }
 };
 
@@ -154,6 +222,20 @@ function removePrefListener(observer)
   }
 }
 
+function addPopupPermListener(observer)
+{
+  const OS = Components.classes["@mozilla.org/observer-service;1"]
+             .getService(Components.interfaces.nsIObserverService);
+  OS.addObserver(observer, "popup-perm-close", false);
+}
+
+function removePopupPermListener(observer)
+{
+  const OS = Components.classes["@mozilla.org/observer-service;1"]
+             .getService(Components.interfaces.nsIObserverService);
+  OS.removeObserver(observer, "popup-perm-close");
+}
+
 /**
 * We can avoid adding multiple load event listeners and save some time by adding
 * one listener that calls all real handlers.
@@ -177,7 +259,7 @@ function loadEventHandlers(event)
 function getContentAreaFrameCount()
 {
   var saveFrameItem = document.getElementById("savepage");
-  if (!_content || !_content.frames.length || !isDocumentFrame(document.commandDispatcher.focusedWindow))
+  if (!content || !content.frames.length || !isContentFrame(document.commandDispatcher.focusedWindow))
     saveFrameItem.setAttribute("hidden", "true");
   else
     saveFrameItem.removeAttribute("hidden");
@@ -187,12 +269,28 @@ function getContentAreaFrameCount()
 function contentAreaFrameFocus()
 {
   var focusedWindow = document.commandDispatcher.focusedWindow;
-  if (isDocumentFrame(focusedWindow)) {
-    gFocusedURL = focusedWindow.location.href;
+  if (isContentFrame(focusedWindow)) {
+    gFocusedURL = Components.lookupMethod(focusedWindow, 'location').call(focusedWindow).href;
     gFocusedDocument = focusedWindow.document;
   }
 }
 
+function updateHomeButtonTooltip()
+{
+  const XUL_NAMESPACE = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
+  var homePage = getHomePage();
+  var tooltip = document.getElementById("home-button-tooltip-inner");
+
+  while (tooltip.firstChild)
+    tooltip.removeChild(tooltip.firstChild);
+
+  for (var i in homePage) {
+    var label = document.createElementNS(XUL_NAMESPACE, "label");
+    label.setAttribute("value", homePage[i]);
+    tooltip.appendChild(label);
+  }
+}
+
 //////////////////////////////// BOOKMARKS ////////////////////////////////////
 
 function UpdateBookmarksLastVisitedDate(event)
@@ -256,18 +354,19 @@ function getBrowser()
 
 function getHomePage()
 {
-  var url;
+  var URIs = [];
   try {
-    url = pref.getComplexValue("browser.startup.homepage",
-                               Components.interfaces.nsIPrefLocalizedString).data;
-  } catch (e) {
+    URIs[0] = pref.getComplexValue("browser.startup.homepage",
+                                   Components.interfaces.nsIPrefLocalizedString).data;
+    var count = pref.getIntPref("browser.startup.homepage.count");
+    for (var i = 1; i < count; ++i) {
+      URIs[i] = pref.getComplexValue("browser.startup.homepage."+i,
+                                     Components.interfaces.nsIPrefLocalizedString).data;
+    }
+  } catch(e) {
   }
 
-  // use this if we can't find the pref
-  if (!url)
-    url = gNavigatorRegionBundle.getString("homePageDefault");
-
-  return url;
+  return URIs;
 }
 
 function UpdateBackForwardButtons()
@@ -303,11 +402,11 @@ function UpdateBackForwardButtons()
 // Used by nsButtonPrefListener to hide the separator if needed
 function allLeftButtonsAreHidden()
 {
-  var buttonNode = document.getElementById("PersonalToolbar").firstChild;
-  while(buttonNode.tagName != "toolbarseparator") {
-    if(!buttonNode.hasAttribute("hidden") || buttonNode.getAttribute("hidden") == "false")
+  var buttonNode = document.getElementById("home-bm-separator").previousSibling;
+  while (buttonNode) {
+    if (buttonNode.localName != "tooltip" && !buttonNode.hidden)
       return false;
-    buttonNode = buttonNode.nextSibling;
+    buttonNode = buttonNode.previousSibling;
   }
   return true;
 }
@@ -324,9 +423,12 @@ function RegisterTabOpenObserver()
     }
   };
 
-  const service = Components.classes["@mozilla.org/observer-service;1"]
+  var service = Components.classes["@mozilla.org/observer-service;1"]
     .getService(Components.interfaces.nsIObserverService);
   service.addObserver(observer, "open-new-tab-request", false);
+  // Null out service variable so the closure of the observer doesn't
+  // own the service and create a cycle (bug 170022).
+  service = null;
 }
 
 function Startup()
@@ -367,9 +469,7 @@ function Startup()
   // Do all UI building here:
 
   // set home button tooltip text
-  var homePage = getHomePage();
-  if (homePage)
-    document.getElementById("home-button").setAttribute("tooltiptext", homePage);
+  updateHomeButtonTooltip();
 
   // initialize observers and listeners
   window.XULBrowserWindow = new nsBrowserStatusHandler();
@@ -377,6 +477,8 @@ function Startup()
   addPrefListener(gButtonPrefListener); 
   addPrefListener(gTabStripPrefListener);
   addPrefListener(gHomepagePrefListener);
+  addPopupPermListener(gPopupPermListener);
+  addPrefListener(gPopupPrefListener);  
 
   window.browserContentListener =
     new nsBrowserContentListener(window, getBrowser());
@@ -445,13 +547,21 @@ function Startup()
 
   // only load url passed in when we're not page cycling
   if (!isPageCycling) {
-    var uriToLoad;
+    var uriToLoad = "";
 
     // Check for window.arguments[0]. If present, use that for uriToLoad.
-    if ("arguments" in window && window.arguments.length >= 1 && window.arguments[0])
-      uriToLoad = window.arguments[0];
+    if ("arguments" in window && window.arguments.length >= 1 && window.arguments[0]) {
+      var uriArray = window.arguments[0].toString().split('\n'); // stringify and split
+      uriToLoad = uriArray.splice(0, 1)[0];
+
+      if (uriArray.length > 0)
+        window.setTimeout(function(arg) { for (var i in arg) gBrowser.addTab(arg[i]); }, 0, uriArray);
+    }
     
-    if (uriToLoad && uriToLoad != "about:blank") {
+    if (/^\s*$/.test(uriToLoad))
+      uriToLoad = "about:blank";
+
+    if (uriToLoad != "about:blank") {
       gURLBar.value = uriToLoad;
       if ("arguments" in window && window.arguments.length >= 3) {
         loadURI(uriToLoad, window.arguments[2]);
@@ -496,8 +606,12 @@ function Startup()
   // initiated by a web page script
   addEventListener("fullscreen", onFullScreen, false);
 
+  addEventListener("PopupWindow", onPopupWindow, false);
+  addEventListener("DOMPopupBlocked", onPopupBlocked, false);
+
   // does clicking on the urlbar select its contents?
   gClickSelectsAll = pref.getBoolPref("browser.urlbar.clickSelectsAll");
+  gClickAtEndSelects = pref.getBoolPref("browser.urlbar.clickAtEndSelects");
 
   // now load bookmarks after a delay
   setTimeout(LoadBookmarksCallback, 0);
@@ -505,16 +619,18 @@ function Startup()
 
 function LoadBookmarksCallback()
 {
-  try {
-    if (!gBookmarksService)
-      gBookmarksService = Components.classes["@mozilla.org/browser/bookmarks-service;1"]
-                                    .getService(Components.interfaces.nsIBookmarksService);
-    gBookmarksService.ReadBookmarks();
-    // tickle personal toolbar to load personal toolbar items
-    var personalToolbar = document.getElementById("NC:PersonalToolbarFolder");
-    personalToolbar.builder.rebuild();
-  } catch (e) {
+  // loads the services
+  initServices();
+  initBMService();
+  var hasRead = BMSVC.readBookmarks();  
+  var bt = document.getElementById("bookmarks-ptf");
+  if (bt) {
+    if (hasRead) 
+      bt.builder.rebuild();
+    bt.database.AddObserver(BookmarksToolbarRDFObserver);
   }
+  window.addEventListener("resize", BookmarksToolbar.resizeFunc, false);
+  controllers.appendController(BookmarksMenuController);
 }
 
 function WindowFocusTimerCallback(element)
@@ -575,6 +691,8 @@ function Shutdown()
   removePrefListener(gButtonPrefListener);
   removePrefListener(gTabStripPrefListener);
   removePrefListener(gHomepagePrefListener);
+  removePopupPermListener(gPopupPermListener);
+  removePrefListener(gPopupPrefListener);
 
   window.browserContentListener.close();
   // Close the app core.
@@ -665,90 +783,14 @@ function BrowserReloadSkipCache()
 function BrowserHome()
 {
   var homePage = getHomePage();
-  loadURI(homePage);
-}
-
-function OpenBookmarkGroup(element, datasource)
-{
-  if (!datasource)
-    return;
-    
-  var id = element.getAttribute("id");
-  var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"]
-                          .getService(Components.interfaces.nsIRDFService);
-  var resource = rdf.GetResource(id, true);
-  OpenBookmarkGroupFromResource(resource, datasource, rdf);
-}
-
-function OpenBookmarkGroupFromResource(resource, datasource, rdf) {
-  var urlResource = rdf.GetResource("http://home.netscape.com/NC-rdf#URL");
-  var rdfContainer = Components.classes["@mozilla.org/rdf/container;1"].getService(Components.interfaces.nsIRDFContainer);
-  rdfContainer.Init(datasource, resource);
-  var containerChildren = rdfContainer.GetElements();
-  var tabPanels = gBrowser.mPanelContainer.childNodes;
-  var tabCount = tabPanels.length;
-  var index = 0;
-  for (; containerChildren.hasMoreElements(); ++index) {
-    var res = containerChildren.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
-    var target = datasource.GetTarget(res, urlResource, true);
-    if (target) {
-      var uri = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
-      gBrowser.addTab(uri);
-    }
-  }  
-  
-  if (index == 0)
-    return; // If the bookmark group was completely invalid, just bail.
-     
-  // Select the first tab in the group if we aren't loading in the background.
-  if (!pref.getBoolPref("browser.tabs.loadInBackground")) {
-    var tabs = gBrowser.mTabContainer.childNodes;
-    gBrowser.selectedTab = tabs[tabCount];
+  if (homePage.length == 1) {
+    loadURI(homePage[0]);
+  } else {
+    for (var i in homePage)
+      gBrowser.addTab(homePage[i]);
   }
 }
 
-function OpenBookmarkURL(node, datasources)
-{
-  if (node.getAttribute("group") == "true")
-    OpenBookmarkGroup(node, datasources);
-    
-  if (node.getAttribute("container") == "true")
-    return;
-
-  var url = node.getAttribute("id");
-  if (!url) // if empty url (most likely a normal menu item like "Manage Bookmarks",
-    return; // don't bother loading it
-  try {
-    // add support for IE favorites under Win32, and NetPositive URLs under BeOS
-    if (datasources) {
-      var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"]
-                          .getService(Components.interfaces.nsIRDFService);
-      var src = rdf.GetResource(url, true);
-      var prop = rdf.GetResource("http://home.netscape.com/NC-rdf#URL", true);
-      var target = datasources.GetTarget(src, prop, true);
-      if (target) {
-        target = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
-        if (target)
-          url = target;
-      }
-    }
-  } catch (ex) {
-    return;
-  }
-
-  // Ignore "NC:" urls.
-  if (url.substring(0, 3) == "NC:")
-    return;
-
-  // Check if we have a browser window
-  if (_content) {
-    loadURI(url);
-    _content.focus();
-  }
-  else
-    openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", url);
-}
-
 function addBookmarkAs()
 {
   const browsers = gBrowser.browsers;
@@ -982,8 +1024,18 @@ function BrowserOpenWindow()
 function BrowserOpenTab()
 {
   if (!gInPrintPreviewMode) {
-    gBrowser.selectedTab = gBrowser.addTab('about:blank');
-    setTimeout("gURLBar.focus();", 0); 
+    var handler = Components.classes['@mozilla.org/commandlinehandler/general-startup;1?type=browser']
+                            .getService(Components.interfaces.nsICmdLineHandler);
+    var uriToLoad = handler.defaultArgs.split("\n")[0];
+    if (/^\s*$/.test(uriToLoad))
+      uriToLoad = "about:blank";
+
+    gBrowser.selectedTab = gBrowser.addTab(uriToLoad);
+    var navBar = document.getElementById("nav-bar");
+    if (uriToLoad == "about:blank" && !navBar.hidden && window.locationbar.visible)
+      setTimeout("gURLBar.focus();", 0);
+    else
+      setTimeout("content.focus();", 0);
   }
 }
 
@@ -1042,13 +1094,37 @@ function BrowserEditBookmarks()
     if (!gDisableBookmarks) {
       gDisableBookmarks = true;
 
-      open("chrome://communicator/content/bookmarks/bookmarks.xul", "_blank",
+      open("chrome://communicator/content/bookmarks/bookmarksManager.xul", "_blank",
         "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar");
       setTimeout(enableBookmarks, 2000);
     }
   }
 }
 
+function updateCloseItems()
+{
+  var browser = getBrowser();
+  if (browser && browser.localName == 'tabbrowser' && browser.getStripVisibility()) {
+    document.getElementById('menu_close').setAttribute('label', gNavigatorBundle.getString('tabs.closeTab'));
+    document.getElementById('menu_closeWindow').hidden = false;
+    document.getElementById('menu_closeOtherTabs').hidden = false;
+    if (browser.mTabContainer.childNodes.length > 1)
+      document.getElementById('cmd_closeOtherTabs').removeAttribute('disabled');
+    else
+      document.getElementById('cmd_closeOtherTabs').setAttribute('disabled', 'true');
+  } else {
+    document.getElementById('menu_close').setAttribute('label', gNavigatorBundle.getString('tabs.close'));
+    document.getElementById('menu_closeWindow').hidden = true;
+    document.getElementById('menu_closeOtherTabs').hidden = true;
+  }
+}
+
+function BrowserCloseOtherTabs()
+{
+  var browser = getBrowser();
+  browser.removeAllTabsBut(browser.mCurrentTab);
+}
+
 function BrowserCloseTabOrWindow()
 {
   var browser = getBrowser();
@@ -1163,12 +1239,20 @@ function BrowserLoadURL(aTriggeringEvent)
       }
     }
     else if (saveModifier) {
-      // Firstly, fixup the url so that (e.g.) "www.foo.com" works
-      const nsIURIFixup = Components.interfaces.nsIURIFixup;
-      var uriFixup = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(nsIURIFixup);
-      url = uriFixup.createFixupURI(url, nsIURIFixup.FIXUP_FLAGS_MAKE_ALTERNATE_URI).spec;
-      // Open filepicker to save the url
-      saveURL(url, "");
+      try {
+        // Firstly, fixup the url so that (e.g.) "www.foo.com" works
+        const nsIURIFixup = Components.interfaces.nsIURIFixup;
+        if (!gURIFixup)
+          gURIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
+                                .getService(nsIURIFixup);
+        url = gURIFixup.createFixupURI(url, nsIURIFixup.FIXUP_FLAGS_MAKE_ALTERNATE_URI).spec;
+        // Open filepicker to save the url
+        saveURL(url, "");
+      }
+      catch(ex) {
+        // XXX Do nothing for now.
+        // Do we want to put up an alert in the future?  Mmm, l10n...
+      }
     }
     else {
       // No modifier was pressed, load the URL normally and
@@ -1312,12 +1396,13 @@ function BrowserViewSourceOfURL(url, charset, pageCookie)
 }
 
 // doc=null for regular page info, doc=owner document for frame info.
-function BrowserPageInfo(doc)
+function BrowserPageInfo(doc, tab)
 {
   window.openDialog("chrome://navigator/content/pageInfo.xul",
                     "_blank",
                     "chrome,dialog=no",
-                    doc);
+                    doc,
+                    tab);
 }
 
 function hiddenWindowStartup()
@@ -1326,7 +1411,7 @@ function hiddenWindowStartup()
   window.focus();
 
   // Disable menus which are not appropriate
-  var disabledItems = ['cmd_close', 'Browser:SendPage', 'Browser:EditPage', 'Browser:PrintSetup', /*'Browser:PrintPreview',*/
+  var disabledItems = ['cmd_close', 'Browser:SendPage', 'Browser:EditPage', 'cmd_printSetup', /*'Browser:PrintPreview',*/
                        'Browser:Print', 'canGoBack', 'canGoForward', 'Browser:Home', 'Browser:AddBookmark', 'cmd_undo',
                        'cmd_redo', 'cmd_cut', 'cmd_copy','cmd_paste', 'cmd_delete', 'cmd_selectAll', 'menu_textZoom'];
   for (var id in disabledItems) {
@@ -1664,7 +1749,8 @@ function URLBarMouseDownHandler(aEvent)
 function URLBarClickHandler(aEvent)
 {
   if (!gIgnoreClick && gClickSelectsAll && gURLBar.selectionStart == gURLBar.selectionEnd)
-    gURLBar.select();
+    if (gClickAtEndSelects || gURLBar.selectionStart < gURLBar.value.length)
+      gURLBar.select();
 }
 
 // This function gets the "windows hooks" service and has it check its setting
@@ -1789,6 +1875,21 @@ function PageProxyDragGesture(aEvent)
   return false;
 }
 
+function handlePageProxyClick(aEvent)
+{
+  switch (aEvent.button) {
+  case 0:
+    // bug 52784 - select location field contents
+    gURLBar.select();
+    break;
+  case 1:
+    // bug 111337 - load url/keyword from clipboard
+    return middleMousePaste(aEvent);
+    break;
+  }
+  return true;
+}
+
 function updateComponentBarBroadcaster()
 { 
   var compBarBroadcaster = document.getElementById('cmd_viewcomponentbar');
@@ -1848,38 +1949,6 @@ function showHideTabbar()
   gBrowser.setStripVisibilityTo(!visibility);
 }
 
-// Fill in tooltips for personal toolbar
-function FillInPTTooltip(tipElement)
-{
-
-  var title = tipElement.label;
-  var url = tipElement.statusText;
-
-  if (!title && !url) {
-    // bail out early if there is nothing to show
-    return false;
-  }
-
-  var tooltipTitle = document.getElementById("ptTitleText");
-  var tooltipUrl = document.getElementById("ptUrlText"); 
-
-  if (title && title != url) {
-    tooltipTitle.removeAttribute("hidden");
-    tooltipTitle.setAttribute("value", title);
-  } else  {
-    tooltipTitle.setAttribute("hidden", "true");
-  }
-
-  if (url) {
-    tooltipUrl.removeAttribute("hidden");
-    tooltipUrl.setAttribute("value", url);
-  } else {
-    tooltipUrl.setAttribute("hidden", "true");
-  }
-
-  return true; // show tooltip
-}
-
 function BrowserFullScreen()
 {
   window.fullScreen = !window.fullScreen;
@@ -1890,6 +1959,104 @@ function onFullScreen()
   FullScreen.toggle();
 }
 
+function onPopupWindow(aEvent) {
+  var firstPopup = pref.getBoolPref("privacy.popups.first_popup");
+  var blockingEnabled = pref.getBoolPref("dom.disable_open_during_load");
+  if (blockingEnabled) {
+    pref.setBoolPref("privacy.popups.first_popup", false);
+    return;
+  }
+  if (firstPopup) { 
+    var showDialog = true;
+    var specialList = "";
+    try {
+      specialList = pref.getComplexValue("privacy.popups.default_whitelist",
+                                         Components.interfaces.nsIPrefLocalizedString).data;
+    }
+    catch(ex) { }
+    if (specialList) {
+      hosts = specialList.split(",");
+      var browser = getBrowserForDocument(aEvent.target);
+      if (!browser)
+        return;
+      var currentHost = browser.currentURI.hostPort;
+      for (var i = 0; i < hosts.length; i++) {
+        var nextHost = hosts[i];
+        if (nextHost == currentHost ||
+            "."+nextHost == currentHost.substr(currentHost.length - (nextHost.length+1))) {
+          showDialog = false;
+          break;
+        }       
+      }
+    }
+    if (showDialog) {
+      window.openDialog("chrome://communicator/content/aboutPopups.xul", "",
+                        "chrome,centerscreen,resizable=yes", true);
+      pref.setBoolPref("privacy.popups.first_popup", false);
+    }
+  }
+}
+
+function onPopupBlocked(aEvent) {
+  var playSound = pref.getBoolPref("privacy.popups.sound_enabled");
+
+  if (playSound) {
+    var sound = Components.classes["@mozilla.org/sound;1"]
+                          .createInstance(Components.interfaces.nsISound);
+
+    var soundUrlSpec = pref.getCharPref("privacy.popups.sound_url");
+
+    if (!soundUrlSpec)
+      sound.beep();
+
+    if (soundUrlSpec.substr(0, 7) == "file://") {
+      var soundUrl = Components.classes["@mozilla.org/network/standard-url;1"]
+                               .createInstance(Components.interfaces.nsIFileURL);
+      soundUrl.spec = soundUrlSpec;
+      var file = soundUrl.file;
+      if (file.exists)
+        sound.play(soundUrl);
+    } 
+    else {
+      sound.playSystemSound(soundUrlSpec);
+    }
+  }
+
+  var showIcon = pref.getBoolPref("privacy.popups.statusbar_icon_enabled");
+  if (showIcon) {
+    var browser = getBrowserForDocument(aEvent.target);      
+    if (browser) {
+      var hostPort = browser.currentURI.hostPort;
+      browser.popupDomain = hostPort;
+      if (browser == getBrowser().selectedBrowser) {
+        var popupIcon = document.getElementById("popupIcon");
+        popupIcon.hidden = false;
+      }    
+    }
+  }
+}
+
+function getBrowserForDocument(doc) {  
+  var browsers = getBrowser().browsers;
+  for (var i = 0; i < browsers.length; i++) {
+    if (browsers[i].contentDocument == doc)
+      return browsers[i];
+  }
+  return null;
+}
+
+function StatusbarViewPopupManager() {
+  var hostPort = "";
+  try {
+    hostPort = getBrowser().selectedBrowser.currentURI.hostPort;
+  }
+  catch(ex) { }
+  
+  // open whitelist with site prefilled to unblock
+  window.openDialog("chrome://communicator/content/popupManager.xul", "",
+                      "chrome,resizable=yes", hostPort, false);
+}
+
 // Set up a lame hack to avoid opening two bookmarks.
 // Could otherwise happen with two Ctrl-B's in a row.
 var gDisableHistory = false;
@@ -1945,3 +2112,34 @@ function checkTheme()
     }
   } 
 }
+
+// opener may not have been initialized by load time (chrome windows only)
+// so call this function some time later.
+function maybeInitPopupContext()
+{
+  // it's not a popup with no opener
+  if (!window.content.opener)
+    return null;
+
+  try {
+    // are we a popup window?
+    const CI = Components.interfaces;
+    var xulwin = window
+                 .QueryInterface(CI.nsIInterfaceRequestor)
+                 .getInterface(CI.nsIWebNavigation)
+                 .QueryInterface(CI.nsIDocShellTreeItem).treeOwner
+                 .QueryInterface(CI.nsIInterfaceRequestor)
+                 .getInterface(CI.nsIXULWindow);
+    if (xulwin.contextFlags &
+        CI.nsIWindowCreator2.PARENT_IS_LOADING_OR_RUNNING_TIMEOUT) {
+      // return our opener's URI
+      const IOS = Components.classes["@mozilla.org/network/io-service;1"]
+                  .getService(CI.nsIIOService);
+      var spec = Components.lookupMethod(window.content.opener, "location")
+                 .call();
+      return IOS.newURI(spec, null, null);
+    }
+  } catch(e) {
+  }
+  return null;
+}
